@@ -35,6 +35,8 @@ class Database:
                 , throttle_period integer
                 , source text
                 , error text
+                , last_started_at timestamptz
+                , last_finished_at timestamptz
                 , created_at timestamptz not null default now()
                 , updated_at timestamptz not null default now()
             );
@@ -50,6 +52,8 @@ class Database:
             comment on column batch_log.throttle_period is 'When throttle_key is set, this specifies the period over which throttle_limit items may be processed';
             comment on column batch_log.source is 'The batch item''s source. Spesifically which component registered it.';
             comment on column batch_log.error is 'The batch item''s error message. Should be None unless status is "failed", in which it case it should be a descriptive error message.';
+            comment on column batch_log.last_started_at is 'When the batch item last started (entered in_progress state)';
+            comment on column batch_log.last_finished_at is 'When the batch item last finished (exited in_progress state)';
             comment on column batch_log.created_at is 'When the batch item was first created';
             comment on column batch_log.updated_at is 'When the batch item was last updated';
             """
@@ -213,8 +217,10 @@ $$ language plpgsql;
                 update
                     batch_log
                 set
-                      status=%(to_status)s
-                    , updated_at=now()
+                      status = %(to_status)s
+                    , updated_at=  now()
+                    , last_finished_at = now()
+                    
                 where
                     status = %(from_status)s
                 and
@@ -247,47 +253,57 @@ $$ language plpgsql;
     # Update status of a batch item in the batch log, making sure to fail if the id and updated_at don't match, providing a guarantee of atomic operation
     # Returns updated_at, so caller can check if it was updated or not (compare it to argument)
     def bump_batch_item(self, id, status, error, updated_at, result=None):
-        if result is None:
-            return self.dbc.query_one(
-                """
-                    update
-                        batch_log
-                    set
-                          status=%(status)s
-                        , error = %(error)s
-                        , updated_at=now()
-                    where
-                        id = %(id)s
-                    and
-                        updated_at = %(updated_at)s
-                    returning
-                          id
-                        , updated_at
-                    ;
-                    """,
-                {"id": (id,), "status": (status,), "error": (error,), "updated_at": (updated_at,)},
-            )
-        else:
-            return self.dbc.query_one(
-                """
-                    update
-                        batch_log
-                    set
-                          status=%(status)s
-                        , error = %(error)s
-                        , result = %(result)s
-                        , updated_at=now()
-                    where
-                        id = %(id)s
-                    and
-                        updated_at = %(updated_at)s
-                    returning
-                          id
-                        , updated_at
-                    ;
-                    """,
-                {"id": (id,), "status": (status,), "error": (error,), "result": (result,), "updated_at": (updated_at,)},
-            )
+        return self.dbc.query_one(
+            """
+                update
+                    batch_log as b
+                set
+                      error = %(error)s
+                    , updated_at = now()
+                    , result =
+                        (case
+                            when
+                                %(result)s is null
+                            then
+                                b.result
+                            else
+                                %(result)s
+                        end)
+                    , last_started_at =
+                        (case
+                            when
+                                b.status is distinct from %(in_progress_status)s
+                            and
+                                %(status)s = %(in_progress_status)s
+                            then
+                                now()
+                            else
+                                b.last_started_at
+                        end)
+
+                    , last_finished_at =
+                        (case
+                            when
+                                b.status = %(in_progress_status)s
+                            and
+                                %(status)s is distinct from %(in_progress_status)s
+                            then
+                                now()
+                            else
+                                b.last_finished_at
+                        end)
+                    , status = %(status)s
+                where
+                    id = %(id)s
+                and
+                    updated_at = %(updated_at)s
+                returning
+                      id
+                    , updated_at
+                ;
+                """,
+            {"id": (id,), "status": (status,), "error": (error,), "result": (result,), "updated_at": (updated_at,), "in_progress_status": (IN_PROGRESS,)},
+        )
 
     # Simplified version of bump_batch_item where status is updated, no questions asked
     def bump_batch_item_status(self, id, status):
